@@ -930,6 +930,7 @@ void CSoundFile::ResetPlayPos()
 	m_PlayState.m_nNextRow = 0;
 	m_PlayState.m_nTickCount = TICKS_ROW_FINISHED;
 	m_PlayState.m_nBufferCount = 0;
+	m_PlayState.m_dBufferDiff = 0;
 	m_PlayState.m_nPatternDelay = 0;
 	m_PlayState.m_nFrameDelay = 0;
 	m_PlayState.m_nextPatStartRow = 0;
@@ -981,6 +982,7 @@ void CSoundFile::SetCurrentOrder(ORDERINDEX nOrder)
 		m_PlayState.m_nPattern = 0;
 		m_PlayState.m_nTickCount = TICKS_ROW_FINISHED;
 		m_PlayState.m_nBufferCount = 0;
+		m_PlayState.m_dBufferDiff = 0;
 		m_PlayState.m_nPatternDelay = 0;
 		m_PlayState.m_nFrameDelay = 0;
 		m_PlayState.m_nextPatStartRow = 0;
@@ -1229,6 +1231,7 @@ PlayBehaviourSet CSoundFile::GetSupportedPlaybackBehaviour(MODTYPE type)
 		playBehaviour.set(kITCarryAfterNoteOff);
 		playBehaviour.set(kITNoteCutWithPorta);
 		playBehaviour.set(kITVolColNoSlidePropagation);
+		playBehaviour.set(kITStoppedFilterEnvAtStart);
 		break;
 
 	case MOD_TYPE_XM:
