@@ -11,8 +11,12 @@
 #include "stdafx.h"
 
 #ifndef NO_PLUGINS
-#include "../../Sndfile.h"
 #include "I3DL2Reverb.h"
+#include "../../Sndfile.h"
+#ifdef MODPLUG_TRACKER
+#include "../../../sounddsp/Reverb.h"
+#endif // MODPLUG_TRACKER
+#include "mpt/base/numbers.hpp"
 #endif // !NO_PLUGINS
 
 OPENMPT_NAMESPACE_BEGIN
@@ -68,15 +72,14 @@ MPT_FORCEINLINE float I3DL2Reverb::DelayLine::Get() const
 }
 
 
-IMixPlugin* I3DL2Reverb::Create(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN *mixStruct)
+IMixPlugin* I3DL2Reverb::Create(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN &mixStruct)
 {
 	return new (std::nothrow) I3DL2Reverb(factory, sndFile, mixStruct);
 }
 
 
-I3DL2Reverb::I3DL2Reverb(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN *mixStruct)
+I3DL2Reverb::I3DL2Reverb(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN &mixStruct)
 	: IMixPlugin(factory, sndFile, mixStruct)
-	, m_recalcParams(true)
 {
 	m_param[kI3DL2ReverbRoom] = 0.9f;
 	m_param[kI3DL2ReverbRoomHF] = 0.99f;
@@ -92,8 +95,9 @@ I3DL2Reverb::I3DL2Reverb(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGI
 	m_param[kI3DL2ReverbHFReference] = (5000.0f - 20.0f) / 19980.0f;
 	m_param[kI3DL2ReverbQuality] = 2.0f / 3.0f;
 
+	SetCurrentProgram(m_program);
+
 	m_mixBuffer.Initialize(2, 2);
-	InsertIntoFactoryList();
 }
 
 
@@ -266,8 +270,8 @@ void I3DL2Reverb::Process(float *pOutL, float *pOutR, uint32 numFrames)
 		float outL = earlyRefOutL + lateRevOutL;
 		float outR = earlyRefOutR + lateRevOutR;
 
-		for(std::size_t d = 0; d < mpt::size(m_delayLines); d++)
-			m_delayLines[d].Advance();
+		for(auto &line : m_delayLines)
+			line.Advance();
 
 		if(!(m_quality & kFullSampleRate))
 		{
@@ -292,6 +296,42 @@ void I3DL2Reverb::Process(float *pOutL, float *pOutR, uint32 numFrames)
 }
 
 
+int32 I3DL2Reverb::GetNumPrograms() const
+{
+#ifdef MODPLUG_TRACKER
+	return NUM_REVERBTYPES;
+#else
+	return 0;
+#endif
+}
+
+void I3DL2Reverb::SetCurrentProgram(int32 program)
+{
+#ifdef MODPLUG_TRACKER
+	if(program < static_cast<int32>(NUM_REVERBTYPES))
+	{
+		m_program = program;
+		const auto &preset = *GetReverbPreset(m_program);
+		m_param[kI3DL2ReverbRoom] = (preset.lRoom + 10000) / 10000.0f;
+		m_param[kI3DL2ReverbRoomHF] = (preset.lRoomHF + 10000) / 10000.0f;
+		m_param[kI3DL2ReverbRoomRolloffFactor] = 0.0f;
+		m_param[kI3DL2ReverbDecayTime] = (preset.flDecayTime - 0.1f) / 19.9f;
+		m_param[kI3DL2ReverbDecayHFRatio] = (preset.flDecayHFRatio - 0.1f) / 1.9f;
+		m_param[kI3DL2ReverbReflections] = (preset.lReflections + 10000) / 11000.0f;
+		m_param[kI3DL2ReverbReflectionsDelay] = preset.flReflectionsDelay / 0.3f;
+		m_param[kI3DL2ReverbReverb] = (preset.lReverb + 10000) / 12000.0f;
+		m_param[kI3DL2ReverbReverbDelay] = preset.flReverbDelay / 0.1f;
+		m_param[kI3DL2ReverbDiffusion] = preset.flDiffusion / 100.0f;
+		m_param[kI3DL2ReverbDensity] = preset.flDensity / 100.0f;
+		m_param[kI3DL2ReverbHFReference] = (5000.0f - 20.0f) / 19980.0f;
+		RecalculateI3DL2ReverbParams();
+	}
+#else
+	MPT_UNUSED_VARIABLE(program);
+#endif
+}
+
+
 PlugParamValue I3DL2Reverb::GetParameter(PlugParamIndex index)
 {
 	if(index < kI3DL2ReverbNumParameters)
@@ -302,13 +342,13 @@ PlugParamValue I3DL2Reverb::GetParameter(PlugParamIndex index)
 }
 
 
-void I3DL2Reverb::SetParameter(PlugParamIndex index, PlugParamValue value)
+void I3DL2Reverb::SetParameter(PlugParamIndex index, PlugParamValue value, PlayState *, CHANNELINDEX)
 {
 	if(index < kI3DL2ReverbNumParameters)
 	{
-		Limit(value, 0.0f, 1.0f);
+		value = mpt::safe_clamp(value, 0.0f, 1.0f);
 		if(index == kI3DL2ReverbQuality)
-			value = Util::Round(value * 3.0f) / 3.0f;
+			value = mpt::round(value * 3.0f) / 3.0f;
 		m_param[index] = value;
 		m_recalcParams = true;
 	}
@@ -353,10 +393,10 @@ void I3DL2Reverb::PositionChanged()
 		m_delayLines[17].Init(10, 0, sampleRate, -1);
 		m_delayLines[18].Init(10, 0, sampleRate, -1);
 		m_ok = true;
-	} MPT_EXCEPTION_CATCH_OUT_OF_MEMORY(e)
+	} catch(mpt::out_of_memory e)
 	{
 		m_ok = false;
-		MPT_EXCEPTION_DELETE_OUT_OF_MEMORY(e);
+		mpt::delete_out_of_memory(e);
 	}
 }
 
@@ -410,7 +450,7 @@ CString I3DL2Reverb::GetParamLabel(PlugParamIndex param)
 
 CString I3DL2Reverb::GetParamDisplay(PlugParamIndex param)
 {
-	static const TCHAR *modes[] = { _T("LQ"), _T("LQ+"), _T("HQ"), _T("HQ+") };
+	static constexpr const TCHAR * const modes[] = { _T("LQ"), _T("LQ+"), _T("HQ"), _T("HQ+") };
 	float value = m_param[param];
 	switch(param)
 	{
@@ -433,6 +473,18 @@ CString I3DL2Reverb::GetParamDisplay(PlugParamIndex param)
 	return s;
 }
 
+
+CString I3DL2Reverb::GetCurrentProgramName()
+{
+	return GetProgramName(m_program);
+}
+
+
+CString I3DL2Reverb::GetProgramName(int32 program)
+{
+	return mpt::ToCString(GetReverbPresetName(program));
+}
+
 #endif // MODPLUG_TRACKER
 
 
@@ -448,12 +500,12 @@ void I3DL2Reverb::RecalculateI3DL2ReverbParams()
 
 	// Room Filter
 	float roomHF = std::pow(10.0f, RoomHF() / 100.0f / 10.0f);
-	if(roomHF == 1.0)
+	if(roomHF == 1.0f)
 	{
 		m_roomFilter = 0.0f;
 	} else
 	{
-		float freq = std::cos(HFReference() * static_cast<float>(2.0 * M_PI) / m_effectiveSampleRate);
+		float freq = std::min(std::cos(HFReference() * (2.0f * mpt::numbers::pi_v<float>) / m_effectiveSampleRate), 0.9999f);
 		float roomFilter = (freq * (roomHF + roomHF) - 2.0f + std::sqrt(freq * (roomHF * roomHF * freq * 4.0f) + roomHF * 8.0f - roomHF * roomHF * 4.0f - roomHF * freq * 8.0f)) / (roomHF + roomHF - 2.0f);
 		m_roomFilter = Clamp(roomFilter, 0.0f, 1.0f);
 	}
@@ -468,7 +520,7 @@ void I3DL2Reverb::RecalculateI3DL2ReverbParams()
 void I3DL2Reverb::SetDelayTaps()
 {
 	// Early reflections
-	static const float delays[] =
+	static constexpr float delays[] =
 	{
 		1.0000f, 1.0000f, 0.0000f, 0.1078f, 0.1768f, 0.2727f,
 		0.3953f, 0.5386f, 0.6899f, 0.8306f, 0.9400f, 0.9800f,
@@ -499,7 +551,7 @@ void I3DL2Reverb::SetDelayTaps()
 	m_delayTaps[13] = static_cast<int32>(3.25f / 1000.0f * sampleRate);
 	m_delayTaps[14] = static_cast<int32>(3.53f / 1000.0f * sampleRate);
 
-	for(std::size_t d = 0; d < mpt::size(m_delayTaps); d++)
+	for(std::size_t d = 0; d < std::size(m_delayTaps); d++)
 		m_delayLines[d].SetDelayTap(m_delayTaps[d]);
 }
 
@@ -555,12 +607,12 @@ void I3DL2Reverb::SetDecayCoeffs()
 
 float I3DL2Reverb::CalcDecayCoeffs(int32 index)
 {
-	float hfRef = static_cast<float>(2.0 * M_PI) / m_effectiveSampleRate * HFReference();
+	float hfRef = (2.0f * mpt::numbers::pi_v<float>) / m_effectiveSampleRate * HFReference();
 	float decayHFRatio = DecayHFRatio();
 	if(decayHFRatio > 1.0f)
-		hfRef = static_cast<float>(M_PI);
+		hfRef = mpt::numbers::pi_v<float>;
 
-	float c1 = std::pow(10.0f, ((m_delayTaps[index] / m_effectiveSampleRate) * -60.0f / DecayTime()) / 20.0f);
+	float c1 = std::pow(10.0f, ((static_cast<float>(m_delayTaps[index]) / m_effectiveSampleRate) * -60.0f / DecayTime()) / 20.0f);
 	float c2 = 0.0f;
 
 	float c21 = (std::pow(c1, 2.0f - 2.0f / decayHFRatio) - 1.0f) / (1.0f - std::cos(hfRef));
@@ -570,8 +622,9 @@ float I3DL2Reverb::CalcDecayCoeffs(int32 index)
 		float c23sq = c22 * c22 - c21 * c21 * 4.0f;
 		float c23 = c23sq > 0.0f ? std::sqrt(c23sq) : 0.0f;
 		c2 = (c23 - c22) / (c21 + c21);
-		if(mpt::abs(c2) > 1.0)
+		if(std::abs(c2) > 1.0f)
 			c2 = (-c22 - c23) / (c21 + c21);
+		c2 = mpt::sanitize_nan(c2);
 	}
 	m_delayCoeffs[index][0] = c1;
 	m_delayCoeffs[index][1] = c2;
